// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:rxdart/rxdart.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/core/exceptions/notification_exeption.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/domain/repositories/cloud_messaging_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class ListenNotificationUseCase
    extends NoParamsAsyncUseCase<Stream<RemoteNotification>> {
  ListenNotificationUseCase({
    required CloudMessagingRepository notificationRepository,
  }) : _notificationRepository = notificationRepository;
  final CloudMessagingRepository _notificationRepository;
  @override
  FutureOrResult<Stream<RemoteNotification>> execute() async {
    Stream<RemoteNotification>? notificationStream;

    final notificationStreamResponse =
        await _notificationRepository.onNotification();

    if (notificationStreamResponse.isOk) {
      notificationStream = notificationStreamResponse.ok;
    } else if (notificationStreamResponse.isErr) {
      if (notificationStreamResponse.err! is NotificationException &&
          (notificationStreamResponse.err! as NotificationException).failure
              is! UnimplementedError) {
        return Err(notificationStreamResponse.err!);
      }

      final notificationBackgroundStreamResponse =
          await _notificationRepository.onNotificationBackground();

      if (notificationBackgroundStreamResponse.isErr) {
        return Err(notificationBackgroundStreamResponse.err!);
      }

      final notificationForegroundStreamResponse =
          await _notificationRepository.onNotificationForeground();

      if (notificationForegroundStreamResponse.isErr) {
        return Err(notificationForegroundStreamResponse.err!);
      }

      notificationStream = Rx.merge([
        notificationForegroundStreamResponse.ok!,
        notificationBackgroundStreamResponse.ok!,
      ]);
    }

    if (notificationStream == null) {
      return Err(const NotificationException('Cannot listen notifications'));
    }

    return Ok(notificationStream);
  }
}
