// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'task_cubit.dart';

abstract class TaskState {
  const TaskState();

  String get description;
}

class TaskStateInitial extends TaskState {
  @override
  String get description => '';
}

class TaskStateCheckTools extends TaskState {
  @override
  String get description => 'Check that needed tools are installed';
}

class TaskStateCheckConfigFile extends TaskState {
  @override
  String get description => 'Check that config file exists';
}

class TaskStateRunning extends TaskState {
  const TaskStateRunning();

  @override
  String get description => '';
}

abstract class TaskStateFinished extends TaskState {
  const TaskStateFinished({
    this.exitCode = 0,
  });

  final int exitCode;
}

class TaskStateError extends TaskStateFinished {
  const TaskStateError({required String? message, super.exitCode = 1})
      : _message = message ?? 'Unknown error';

  factory TaskStateError.unknown() => const TaskStateError(message: null);

  factory TaskStateError.fromException(AppException e, {int? exitCode}) =>
      TaskStateError(message: e.message, exitCode: exitCode ?? 1);

  factory TaskStateError.fromRes(
    Result<dynamic, AppException> res, {
    int? exitCode,
  }) =>
      TaskStateError(message: res.err?.message, exitCode: exitCode ?? 1);

  final String _message;

  @override
  String get description =>
      'An error occured${_message.isNotEmpty ? ': $_message' : ''}';
}

class TaskStateSucceed extends TaskStateFinished {
  const TaskStateSucceed({this.message, super.exitCode});

  final String? message;

  @override
  String get description =>
      message ?? 'The task ended with success (exit code: $exitCode)';
}
