// Copyright (C) 2024 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_continuous_deployment/src/domain/entities/config/ios_config.dart';

class AppStoreConnectConfig {
  AppStoreConnectConfig({
    required this.keyId,
    required this.issuerId,
    required this.keyFilePath,
    required this.isKeyContentBase64,
    required this.duration,
    required this.inHouse,
  });

  factory AppStoreConnectConfig.fromJson(Map<String, dynamic> json) =>
      AppStoreConnectConfig(
        keyId: json['apple_key_id']?.toString(),
        issuerId: json['apple_issuer_id']?.toString(),
        keyFilePath: json['apple_key_content']?.toString(),
        isKeyContentBase64: false,
        duration: 500,
        inHouse: false,
      );

  factory AppStoreConnectConfig.fromConfig(IosConfig config) =>
      AppStoreConnectConfig(
        keyId: config.appleKeyId,
        issuerId: config.appleIssuerId,
        keyFilePath:
            '${config.iosFolderPath}/${config.appleApiPrivateKeyFileName}',
        isKeyContentBase64: false,
        duration: 500,
        inHouse: false,
      );

  final String? keyId;
  final String? issuerId;
  final String? keyFilePath;
  final bool? isKeyContentBase64;
  final int? duration;
  final bool? inHouse;

  Map<String, dynamic> toJson(String key) => {
        'key_id': keyId,
        'issuer_id': issuerId,
        'key': key,
        'is_key_content_base64': isKeyContentBase64,
        'duration': duration,
        'in_house': inHouse,
      };
}
